/*
  KeePT - Plugin for KeePass 1.21
  Copyright (C) 2010 Nikolaus Hammler <niki@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

// TODO: Make a proxy for _MVC functions
#define _CRT_SECURE_NO_WARNINGS

#include <assert.h>
#include <tchar.h>
#include <ctype.h>
#include "KeePTHotKeys.h"
#include "KeePTPlugin.h"
#include "WinPTExec.h"

extern IKpAPI3 *g_pAPI;

static PTHOTKEY g_HotKeys[NUM_HOTKEYS] = {
	{ "", 0, MOD_CONTROL | MOD_ALT, 0, 0, "46261F6F-838E-4621-BD28-A40F129CD206" },
	// Clipboard operations
	{ "ClipDecrypt", 0, MOD_CONTROL	| MOD_ALT, 0, ID_WINPT_DECRYPT_VERIFY, "4E3B9869-9009-4b8f-80E0-6D99A134BF27" },
	{ "ClipEncrypt", 0, MOD_CONTROL	| MOD_ALT, 0, ID_WINPT_ENCRYPT, "F41C9EC3-FC68-4e3b-B81D-A1E3CA8C8DA3" },
	{ "ClipSign", 0, MOD_CONTROL | MOD_ALT, 0, ID_WINPT_SIGN, "86CEF3BC-A9F9-452f-88CA-9BFB1C1B637C" },
	{ "ClipSignEnc", 0, MOD_CONTROL	| MOD_ALT, 0, ID_WINPT_SIGNENCRYPT, "D8B6D392-19E9-442c-820F-71733A942D2F" },
	// current window operations
	{ "CwsDecrypt", 0, MOD_CONTROL | MOD_SHIFT, 0, ID_WINPT_CURRWND_DECRYPT_VERIFY, "2AA321C6-FDE0-46aa-8510-F2FD892795BA" },
	{ "CwsEncrypt", 0, MOD_CONTROL | MOD_SHIFT, 0, ID_WINPT_CURRWND_ENCRYPT, "E5E19F9B-410A-42f4-A99A-5175400510A6" },
	{ "CwsSign", 0, MOD_CONTROL | MOD_SHIFT, 0, ID_WINPT_CURRWND_SIGN, "B25F553F-F2B6-4264-8E60-D0BC1285AAC2" },
	{ "CwsSignEnc", 0, MOD_CONTROL | MOD_SHIFT, 0, ID_WINPT_CURRWND_SIGNENCRYPT, "ECD7E866-59D5-4f1b-816D-394CE5EFFC4E" },
};

bool OnWmHotKey(int idHotKey, UINT fuModifiers, UINT uVirtKey)
{
	if(idHotKey == g_HotKeys[0].id)
	{
		MessageBox(g_pAPI->GetMainWindowHandle(), _T("This hot-key is not supported by KeePT"), MSGTITLE, MB_OK | MB_ICONASTERISK);
		return true;
	}

	for(int i = 1; i < NUM_HOTKEYS; i++)
	{
		if(g_HotKeys[i].id == idHotKey)
		{
			ExecuteWinPT(g_HotKeys[i].dwCmd);
			return true;
		}
	}
	return false;
}

void UnregisterHotKeys(HWND hWnd)
{
	for(int i = 0; i < NUM_HOTKEYS; i++)
	{
		if(g_HotKeys[i].id)
		{
			UnregisterHotKey(hWnd, g_HotKeys[i].id);
			GlobalDeleteAtom(g_HotKeys[i].id);
			g_HotKeys[i].id = 0;
			g_HotKeys[i].vKey = 0;
		}
	}
}

bool UpdateHotKeys(HWND hWnd)
{
	HKEY hRegKey = NULL;
	BYTE regValue[SI_REGSIZE];
	DWORD regSize = SI_REGSIZE-1;
	DWORD regType = 0;

	IKpConfig *pCfg = NULL;
	if(g_pAPI->CreateInstance(CLSID_KpConfig_ReadOnly, IID_IKpConfig, (void**)&pCfg) != S_OK)
		return false;

	if(!pCfg->GetBool(CONF_WPTHKEY, CONF_WPTHKEY_DEF))
	{
		UnregisterHotKeys(hWnd);
		pCfg->Release();
		return false;
	}
	pCfg->Release(); pCfg = NULL;

	if(RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\WinPT"), 0, KEY_READ, &hRegKey) != ERROR_SUCCESS)
	{
		MessageBox(g_pAPI->GetMainWindowHandle(), _T("KeePT settings requested to install HotKeys but registry key HKCU\\Software\\WinPT could not be opened."), MSGTITLE, MB_OK | MB_ICONERROR);
		UnregisterHotKeys(hWnd);
		return false;
	}

	if(RegQueryValueEx(hRegKey, _T("DisableHotkeys"), NULL, &regType, regValue, &regSize) == ERROR_SUCCESS)
	{
		if(atoi((const char*)regValue) == 1)
		{
			UnregisterHotKeys(hWnd);
			RegCloseKey(hRegKey);
			return true;
		}
	}

	DWORD failed = 0;

	// this one is always active ...
	if(!g_HotKeys[0].id)
	{
		g_HotKeys[0].id = GlobalAddAtom(g_HotKeys[0].guid);
		g_HotKeys[0].vKey = 'F';
		if(!(g_HotKeys[0].id && RegisterHotKey(hWnd, g_HotKeys[0].id, g_HotKeys[0].fsModifiers, g_HotKeys[0].vKey)))
		{
			failed = 1;
		}
	}

	for(int i = 1; i < NUM_HOTKEYS; i++)
	{
		int vKey = 0;
		if(RegQueryValueEx(hRegKey, g_HotKeys[i].regKey, NULL, &regType, regValue, &regSize) == ERROR_SUCCESS)
		{
			if(regType == REG_SZ && isalnum(regValue[0]))
			{
				vKey = regValue[0];
			}
		}

		if(vKey)
		{
			// if g_HotKeys[i].vKey = vKey, we assume we have the same hot key already set
			// Do not execute this stuff again, otherweise we get an already registered error
			// Not that g_HotKeys[i].vKey is only set when hot key is active
			if(g_HotKeys[i].vKey != vKey)
			{
				g_HotKeys[i].vKey = vKey;

				// If not yet set, create ID for hotkey
				if(!g_HotKeys[i].id)
					g_HotKeys[i].id = GlobalAddAtom(g_HotKeys[i].guid);

				if(!(g_HotKeys[i].id && RegisterHotKey(hWnd, g_HotKeys[i].id, g_HotKeys[i].fsModifiers, g_HotKeys[i].vKey)))
				{
					failed |= 1 << i;
				}
			}
		}
		else
		{
			// if NOT key set (any more?) and previously existed,
			// remove hot key
			if(g_HotKeys[i].id)
			{
				UnregisterHotKey(hWnd, g_HotKeys[i].id);
				GlobalDeleteAtom(g_HotKeys[i].id);
				g_HotKeys[i].id = 0;
				g_HotKeys[i].vKey = 0;
			}
		}
	}
	RegCloseKey(hRegKey);

	if(failed)
	{
		char msg[2048] = _T("Failed to register the following hot keys for KeePT:\n");
		for(int i = 0; i < NUM_HOTKEYS; i++)
		{
			if(failed & (1 << i))
			{
				char key[] = { g_HotKeys[i].vKey, '\0' };
				if((g_HotKeys[i].fsModifiers & MOD_CONTROL) == MOD_CONTROL)
					strcat(msg, _T("CTRL + "));
				if((g_HotKeys[i].fsModifiers & MOD_ALT) == MOD_ALT)
					strcat(msg, _T("ALT + "));
				if((g_HotKeys[i].fsModifiers & MOD_SHIFT) == MOD_SHIFT)
					strcat(msg, _T("SHIFT + "));
				strncat(msg, key, 2);
				strcat(msg, "\n");
			}
		}
		MessageBox(g_pAPI->GetMainWindowHandle(), msg, MSGTITLE, MB_OK | MB_ICONWARNING);
	}

	return true;
}


